@@ -1,6 +1,6 @@
 # Soft Voting/Majority Rule classifier
 
-# Sebastian Raschka 2014-2022
+# Sebastian Raschka 2014-2023
 # mlxtend Machine Learning Library Extensions
 #
 # Implementation of an meta-classification algorithm for majority voting.
@@ -105,7 +105,7 @@ class EnsembleVoteClassifier(BaseEstimator, ClassifierMixin, TransformerMixin):
     >>>
 
     For more usage examples, please see
-    http://rasbt.github.io/mlxtend/user_guide/classifier/EnsembleVoteClassifier/
+    https://rasbt.github.io/mlxtend/user_guide/classifier/EnsembleVoteClassifier/
     """
 
     def __init__(
@@ -117,7 +117,6 @@ class EnsembleVoteClassifier(BaseEstimator, ClassifierMixin, TransformerMixin):
         use_clones=True,
         fit_base_estimators=True,
     ):
-
         self.clfs = clfs
         self.named_clfs = {key: value for key, value in _name_estimators(clfs)}
         self.voting = voting
@@ -169,7 +168,7 @@ class EnsembleVoteClassifier(BaseEstimator, ClassifierMixin, TransformerMixin):
         self.le_.fit(y)
         self.classes_ = self.le_.classes_
 
-        if not self.fit_base_estimators:
+        if not self.fit_base_estimators and self.use_clones:
             warnings.warn(
                 "fit_base_estimators=False " "enforces use_clones to be `False`"
             )
@@ -185,7 +184,6 @@ class EnsembleVoteClassifier(BaseEstimator, ClassifierMixin, TransformerMixin):
                 print("Fitting %d classifiers..." % (len(self.clfs)))
 
             for clf in self.clfs_:
-
                 if self.verbose > 0:
                     i = self.clfs_.index(clf) + 1
                     print(
@@ -227,7 +225,6 @@ class EnsembleVoteClassifier(BaseEstimator, ClassifierMixin, TransformerMixin):
             )
 
         if self.voting == "soft":
-
             maj = np.argmax(self.predict_proba(X), axis=1)
 
         else:  # 'hard' voting
