@@ -1,6 +1,6 @@
 # Stacking CV classifier
 
-# Sebastian Raschka 2014-2022
+# Sebastian Raschka 2014-2023
 # mlxtend Machine Learning Library Extensions
 #
 # An ensemble-learning meta-classifier for stacking
@@ -106,8 +106,8 @@ class StackingCVClassifier(
         to scikit-learn's `clone` function.
     n_jobs : int or None, optional (default=None)
         The number of CPUs to use to do the computation.
-        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
-        ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
+        `None` means 1 unless in a `joblib.parallel_backend` context.
+        `-1` means using all processors.
         for more details. New in v0.16.0.
     pre_dispatch : int, or string, optional
         Controls the number of jobs that get dispatched during parallel
@@ -138,7 +138,7 @@ class StackingCVClassifier(
     Examples
     -----------
     For usage examples, please see
-    http://rasbt.github.io/mlxtend/user_guide/classifier/StackingCVClassifier/
+    https://rasbt.github.io/mlxtend/user_guide/classifier/StackingCVClassifier/
 
     """
 
@@ -159,7 +159,6 @@ class StackingCVClassifier(
         n_jobs=None,
         pre_dispatch="2*n_jobs",
     ):
-
         self.classifiers = classifiers
         self.meta_classifier = meta_classifier
         self.use_probas = use_probas
@@ -243,7 +242,6 @@ class StackingCVClassifier(
         meta_features = None
 
         for n, model in enumerate(self.clfs_):
-
             if self.verbose > 0:
                 i = self.clfs_.index(model) + 1
                 print(
