@@ -1,4 +1,4 @@
-# Sebastian Raschka 2014-2022
+# Sebastian Raschka 2014-2023
 # mlxtend Machine Learning Library Extensions
 #
 # Feature Importance Estimation Through Permutation
@@ -10,7 +10,7 @@ import numpy as np
 
 
 def feature_importance_permutation(
-    X, y, predict_method, metric, num_rounds=1, seed=None
+    X, y, predict_method, metric, num_rounds=1, feature_groups=None, seed=None
 ):
     """Feature importance imputation via permutation importance
 
@@ -41,6 +41,12 @@ def feature_importance_permutation(
         Number of rounds the feature columns are permuted to
         compute the permutation importance.
 
+    feature_groups : list or None (default=None)
+        Optional argument for treating certain features as a group.
+        For example `[1, 2, [3, 4, 5]]`, which can be useful for
+        interpretability, for example, if features 3, 4, 5 are one-hot
+        encoded features.
+
     seed : int or None (default=None)
         Random seed for permuting the feature columns.
 
@@ -57,7 +63,7 @@ def feature_importance_permutation(
     Examples
     -----------
     For usage examples, please see
-    http://rasbt.github.io/mlxtend/user_guide/evaluate/feature_importance_permutation/
+    https://rasbt.github.io/mlxtend/user_guide/evaluate/feature_importance_permutation/
 
     """
 
@@ -90,20 +96,42 @@ def feature_importance_permutation(
 
     rng = np.random.RandomState(seed)
 
-    mean_importance_vals = np.zeros(X.shape[1])
-    all_importance_vals = np.zeros((X.shape[1], num_rounds))
-
     baseline = score_func(y, predict_method(X))
 
-    for round_idx in range(num_rounds):
-        for col_idx in range(X.shape[1]):
-            save_col = X[:, col_idx].copy()
-            rng.shuffle(X[:, col_idx])
-            new_score = score_func(y, predict_method(X))
-            X[:, col_idx] = save_col
-            importance = baseline - new_score
-            mean_importance_vals[col_idx] += importance
-            all_importance_vals[col_idx, round_idx] = importance
-    mean_importance_vals /= num_rounds
+    if feature_groups is None:
+        mean_importance_vals = np.zeros(X.shape[1])
+        all_importance_vals = np.zeros((X.shape[1], num_rounds))
+
+        for round_idx in range(num_rounds):
+            for col_idx in range(X.shape[1]):
+                save_col = X[:, col_idx].copy()
+                rng.shuffle(X[:, col_idx])
+                new_score = score_func(y, predict_method(X))
+                X[:, col_idx] = save_col
+                importance = baseline - new_score
+                mean_importance_vals[col_idx] += importance
+                all_importance_vals[col_idx, round_idx] = importance
+        mean_importance_vals /= num_rounds
+
+    else:
+        mean_importance_vals = np.zeros(len(feature_groups))
+        all_importance_vals = np.zeros((len(feature_groups), num_rounds))
+        for round_idx in range(num_rounds):
+            for col_idx, feat in enumerate(feature_groups):
+                save_col = X[:, feat].copy()
+
+                if save_col.ndim > 1:
+                    columns = save_col.shape[1]
+                    for i in range(columns):
+                        rng.shuffle(X[:, i])
+                else:
+                    rng.shuffle(X[:, feat])
+
+                new_score = score_func(y, predict_method(X))
+                X[:, feat] = save_col
+                importance = baseline - new_score
+                mean_importance_vals[col_idx] += importance
+                all_importance_vals[col_idx, round_idx] = importance
+        mean_importance_vals /= num_rounds
 
     return mean_importance_vals, all_importance_vals
