@@ -1,4 +1,4 @@
-# Sebastian Raschka 2014-2022
+# Sebastian Raschka 2014-2023
 # mlxtend Machine Learning Library Extensions
 #
 # Base Classifier (Classifier Parent Class)
@@ -23,7 +23,7 @@ class _BaseFeatureExtractor(object):
         try:
             if y is None:
                 return
-        except (AttributeError):
+        except AttributeError:
             if not len(y.shape) == 1:
                 raise ValueError("y must be a 1D array.")
 
