@@ -1,4 +1,4 @@
-# Sebastian Raschka 2014-2022
+# Sebastian Raschka 2014-2023
 # mlxtend Machine Learning Library Extensions
 #
 # Principal Component Analysis for dimensionality reduction.
@@ -52,14 +52,14 @@ class PrincipalComponentAnalysis(_BaseModel):
     Examples
     -----------
     For usage examples, please see
-    http://rasbt.github.io/mlxtend/user_guide/feature_extraction/PrincipalComponentAnalysis/
+    https://rasbt.github.io/mlxtend/user_guide/feature_extraction/PrincipalComponentAnalysis/
 
     """
 
     def __init__(self, n_components=None, solver="svd", whitening=False):
         valid_solver = {"eigen", "svd"}
         if solver not in valid_solver:
-            raise AttributeError("Must be in %s. Found %s" % (valid_solver, solver))
+            raise AttributeError(f"Must be in {valid_solver}. Found {solver}")
         self.solver = solver
 
         if n_components is not None and n_components < 1:
