@@ -1,4 +1,4 @@
-# Sebastian Raschka 2014-2022
+# Sebastian Raschka 2014-2023
 # mlxtend Machine Learning Library Extensions
 #
 # Function for plotting linear regression fits via scikit-learn and matplotlib.
@@ -58,13 +58,13 @@ def plot_linear_regression(
     Examples
     -----------
     For usage examples, please see
-    http://rasbt.github.io/mlxtend/user_guide/plotting/plot_linear_regression/
+    https://rasbt.github.io/mlxtend/user_guide/plotting/plot_linear_regression/
 
     """
     if isinstance(X, list):
-        X = np.asarray(X, dtype=np.float)
+        X = np.asarray(X, dtype=float)
     if isinstance(y, list):
-        y = np.asarray(y, dtype=np.float)
+        y = np.asarray(y, dtype=float)
     if len(X.shape) == 1:
         X = X[:, np.newaxis]
 
